# frozen_string_literal: true

require 'cht-chaphql'
require 'cht-chaphql/graphql_formatters'

RIGHTSIZING_FRAMEWORK_FMTRS =
  YAML.load_file(File.join(__dir__, '../helpers/formatters.yml'))

# module for rightsizing-framework
module ChtApitestRightsizingFramework
  include Chaphql
  include Chaphql::GraphQLHelpers

  # module for your shortcuts
  module RightsizingFrameworkHelpers
    def via_public
      %w[staging canary prod].include?(getenv)
    end

    def auth_service
      LocalAuthService.new(via_public: via_public)
    end
    # insertion point for shortcuts

    def create_rightsizing_efficiency_target(args = {})
      args[:service] = RightsizingFrameworkService.new
      sym = first_no_cap(camelize('create_rightsizing_efficiency_target')).to_sym
      headers = apitest_headers(
        args.merge(with_auth: true,
                   auth_service: auth_service))
      data = args[:body] || format_request(sym, args)
      variables = {"input"=>{"assetType"=>args[:assetType], "metrics"=>args[:metrics], "name"=>args[:name]}}
      graphql url(args), data, variables, headers, args
      expect_success_graphql
      check(sym, args)
      json_body[:data][sym]
    end

    def update_rightsizing_efficiency_target(args = {})
      args[:service] = RightsizingFrameworkService.new
      sym = first_no_cap(camelize('update_rightsizing_efficiency_target')).to_sym
      headers = apitest_headers(
        args.merge(with_auth: true,
                   auth_service: auth_service))
      data = args[:body] || format_request(sym, args)
      variables = {"input"=>{"id"=>args[:id], "metrics"=>args[:metrics], "name"=>args[:name]}}
      graphql url(args), data, variables, headers, args
      expect_success_graphql
      check(sym, args)
      json_body[:data][sym]
    end

    def delete(args = {})
      args[:service] = RightsizingFrameworkService.new
      sym = first_no_cap(camelize('delete')).to_sym
      headers = apitest_headers(
        args.merge(with_auth: true,
                   auth_service: auth_service))
      data = args[:body] || format_request(sym, args)
      variables = {"id"=>args[:id]}
      graphql url(args), data, variables, headers, args
      expect_success_graphql
      check(sym, args)
      json_body[:data][sym]
    end

    def rightsizing_recommendations(args = {})
      args[:service] = RightsizingFrameworkService.new
      sym = first_no_cap(camelize('rightsizing_recommendations')).to_sym
      headers = apitest_headers(
        args.merge(with_auth: true,
                   auth_service: auth_service))
      data = args[:body] || format_request(sym, args)
      variables = {"requestInput"=>{"assetType"=>args[:assetType], "efficiencyTarget"=>args[:efficiencyTarget], "evaluationDuration"=>args[:evaluationDuration], "filterOptions"=>args[:filterOptions], "processingFlagsMap"=>args[:processingFlagsMap], "returnAllInstances"=>args[:returnAllInstances]},"after"=>args[:after], "first"=>args[:first],"sortRules"=>args[:sortRules]}
      graphql url(args), data, variables, headers, args
      expect_success_graphql
      check(sym, args)
      json_body[:data][sym]
    end

    def rightsizing_recommendations_summary(args = {})
      args[:service] = RightsizingFrameworkService.new
      sym = first_no_cap(camelize('rightsizing_recommendations_summary')).to_sym
      headers = apitest_headers(
        args.merge(with_auth: true,
                   auth_service: auth_service))
      data = args[:body] || format_request(sym, args)
      variables = {"request"=>{"assetType"=>args[:assetType], "efficiencyTarget"=>args[:efficiencyTarget], "evaluationDuration"=>args[:evaluationDuration], "filterOptions"=>args[:filterOptions], "processingFlagsMap"=>args[:processingFlagsMap]}}
      graphql url(args), data, variables, headers, args
      expect_success_graphql
      json_body[:data][sym]
    end

    def rightsizing_recommendations_aggregation(args = {})
      args[:service] = RightsizingFrameworkService.new
      sym = first_no_cap(camelize('rightsizing_recommendations_aggregation')).to_sym
      headers = apitest_headers(
        args.merge(with_auth: true,
                   auth_service: auth_service))
      data = args[:body] || format_request(sym, args)
      variables = {"requestInput"=>{"assetType"=>args[:assetType], "efficiencyTarget"=>args[:efficiencyTarget], "evaluationDuration"=>args[:evaluationDuration], "filterOptions"=>args[:filterOptions], "processingFlagsMap"=>args[:processingFlagsMap], "returnAllInstances"=>args[:returnAllInstances], "groupingCriteria"=>args[:groupingCriteria]},"sortRules"=>args[:sortRules]}
      graphql url(args), data, variables, headers, args
      expect_success_graphql
      check(sym, args)
      json_body[:data][sym]
    end

    def rightsizing_efficiency_targets(args = {})
      args[:service] = RightsizingFrameworkService.new
      sym = first_no_cap(camelize('rightsizing_efficiency_targets')).to_sym
      headers = apitest_headers(
        args.merge(with_auth: true,
                   auth_service: auth_service))
      data = args[:body] || format_request(sym, args)
      variables = {"assetType"=>args[:assetType]}
      graphql url(args), data, variables, headers, args
      expect_success_graphql
      check(sym, args)
      json_body[:data][sym]
    end

    def node(args = {})
      args[:service] = RightsizingFrameworkService.new
      sym = first_no_cap(camelize('node')).to_sym
      headers = apitest_headers(
        args.merge(with_auth: true,
                   auth_service: auth_service))
      data = args[:body] || format_request(sym, args)
      variables = {"id"=>args[:id]}
      graphql url(args), data, variables, headers, args
      expect_success_graphql
      check(sym, args) unless args[:body]
      json_body[:data][sym]
    end

    def _service(args = {})
      args[:service] = RightsizingFrameworkService.new
      sym = first_no_cap(camelize('_service')).to_sym
      headers = apitest_headers(
        args.merge(with_auth: true,
                   auth_service: auth_service))
      data = args[:body] || format_request(sym, args)
      variables = {}
      graphql url(args), data, variables, headers, args
      expect_success_graphql
      check(sym, args) unless args[:body]
      json_body[:data][sym]
    end

    def _service(args = {})
      args[:service] = RightsizingFrameworkService.new
      sym = first_no_cap(camelize('_service')).to_sym
      headers = apitest_headers(
        args.merge(with_auth: true,
                   auth_service: auth_service))
      data = args[:body] || format_request(sym, args)
      variables = {}
      graphql url(args), data, variables, headers, args
      expect_success_graphql
      # check(sym, args)
      json_body[:data][sym]
    end

    # end of shortcut insertion
  end

  # class for your service
  class RightsizingFrameworkService < ChtNgService
    include RSpec::Matchers

    RIGHTSIZING_FRAMEWORK_DEFAULTS = {
      name: 'RightsizingFramework',
      default_namespace: 'rightsizing-framework',
      service_name: 'rightsizing-framework',
      pr_parent: 'rightsizing-framework',
      formatters: RIGHTSIZING_FRAMEWORK_FMTRS['formatters'],
      v2_checkers: RIGHTSIZING_FRAMEWORK_FMTRS['v2_checkers']
    }.freeze
    def initialize(args = {})
      super(args.merge(RIGHTSIZING_FRAMEWORK_DEFAULTS))
    end
  end

  # class for api-proxy support of rightsizing-framework service
  class ApiProxyRightsizingFramework < ChtNgService
    include RSpec::Matchers

    API_PROXY_DEFAULTS = {
      name: 'ApiProxyRightsizingFramework',
      default_namespace: 'api-proxy',
      service_name: 'api-proxy',
      pr_parent: 'api-proxy',
      formatters: RIGHTSIZING_FRAMEWORK_FMTRS['formatters'],
      v2_checkers: RIGHTSIZING_FRAMEWORK_FMTRS['v2_checkers']
    }.freeze
    def initialize(args = {})
      super(args.merge(API_PROXY_DEFAULTS))
    end
  end
end
