require 'spec_helper'
require 'date'

module RightsizingFrameworkTest
  describe 'RightsizingFramework GetRecommendations' do
    include_context 'request metadata'

    # test -1 Validate show all
      it 'EBS Validate show ALL is false', monitoring: true do
        args_get_recommendations = {
          assetType: 'AWS_EBS',
          efficiencyTarget: 'crn:0:rightsizing/AWS_EBS_EfficiencyTarget:system_avg',
          evaluationDuration: 'LAST_30_DAYS',
          returnAllInstances: false
        }
        response_recommendations_call = rightsizing_recommendations(args_get_recommendations)
        expect(response_recommendations_call[:edges].all? { |r| r[:node][:options].length > 0 }).to be true
      end

    #test -2 validate show ALL true
      xit 'EBS Validate show ALL is true' do
        args_get_recommendations = {
          assetType: 'AWS_EBS',
          efficiencyTarget: 'crn:0:rightsizing/AWS_EBS_EfficiencyTarget:system_avg',
          evaluationDuration: 'LAST_30_DAYS',
          returnAllInstances: true,
          after: nil
        }
        response_recommendations_call = rightsizing_recommendations(args_get_recommendations)
        count = response_recommendations_call[:edges].select { |x| x[:node][:options].length == 0 }.length
        while(response_recommendations_call[:pageInfo][:hasNextPage]) do
          args_get_recommendations[:after] = response_recommendations_call[:pageInfo][:endCursor]
          response_recommendations_call = rightsizing_recommendations(args_get_recommendations)
          count += response_recommendations_call[:edges].select { |x| x[:node][:options].length == 0 }.length
        end
        expect(count).not_to eql(0)
      end

    # Test -3 : Validate best recommendation price is the lowest among the recommendation options
      xit 'EBS Validate the first best recommendation is of the lowest cost among the recommendations' do
        args_get_recommendations = {
          assetType: 'AWS_EBS',
          efficiencyTarget: 'crn:0:rightsizing/AWS_EBS_EfficiencyTarget:system_avg',
          evaluationDuration: 'LAST_30_DAYS',
          returnAllInstances: false
        }
        response_recommendations = rightsizing_recommendations(args_get_recommendations)
        response_recommendations[:edges].each do |edge|
          if (edge[:node][:terminateRecommendation]).eql? false
            array_of_projectedmonthly=edge[:node][:options].map {|o| o[:projectedMonthlyPrice]}
            projectedMonthlyPrice = edge[:node][:options].select { |o| o[:projectedMonthlyPrice] if o[:bestFit].eql?(true) }[0][:projectedMonthlyPrice]
            expect(projectedMonthlyPrice).to eq(array_of_projectedmonthly.min)
          end
        end
      end

    #  Test - 4 : Validate that the savings is the difference of the current and the projected price
      xit 'EBS Validate saving of an instance is the difference of current price and the estimated price' do
        args_get_recommendations = {
          assetType: 'AWS_EBS',
          efficiencyTarget: 'crn:0:rightsizing/AWS_EBS_EfficiencyTarget:system_avg',
          evaluationDuration: 'LAST_30_DAYS',
          returnAllInstances: false
        }
        response_recommendations = rightsizing_recommendations(args_get_recommendations)
        response_recommendations[:edges].each do |edge|
          expect((edge[:node][:currentMonthlyPrice]- edge[:node][:projectedMonthlyPrice]).round(2).eql?(edge[:node][:projectedMonthlySavings].round(2))).to be true
        end
        end

    # test -5 termination recommendation the current monthly price equal saving and no projected monthly price
      xit 'EBS Validate in termination recommendation the current monthly price equal saving and no projected monthly price' do
        args_get_recommendations = {
          assetType: 'AWS_EBS',
          efficiencyTarget: 'crn:0:rightsizing/AWS_EBS_EfficiencyTarget:system_avg',
          evaluationDuration: 'LAST_30_DAYS',
          returnAllInstances: false
        }
        response_recommendations = rightsizing_recommendations(args_get_recommendations)
        response_recommendations[:edges].each do |edge|
          if (edge[:node][:terminateRecommendation])
            expect((edge[:node][:currentMonthlyPrice]).eql?(edge[:node][:projectedMonthlySavings]) && edge[:node][:projectedMonthlyPrice].eql?(0.0)).to be true
          end
        end
      end

    #test - 6 Validate termination recommendation
    xit 'EBS Validate in termination recommendation logic' do
      args_get_recommendations = {
        assetType: 'AWS_EBS',
        efficiencyTarget: 'crn:0:rightsizing/AWS_EBS_EfficiencyTarget:system_avg',
        evaluationDuration: 'LAST_30_DAYS',
        returnAllInstances: false
      }
      response_recommendations = rightsizing_recommendations(args_get_recommendations)
      response_recommendations[:edges].each do |edge|
        if (edge[:node][:readOps]===0 && edge[:node][:writeOps]===0)
          expect(edge[:node][:terminateRecommendation]).to be true
        end
      end
    end

    #test - 6 Validate no recommendation
    xit 'EBS Validate current monthly price, saving and projected monthly price in case of not enough data' do
      args_get_recommendations = {
        assetType: 'AWS_EBS',
        efficiencyTarget: 'crn:0:rightsizing/AWS_EBS_EfficiencyTarget:system_avg',
        evaluationDuration: 'LAST_30_DAYS',
        returnAllInstances: true,
        filterOptions:{
          otherCriteria:[
            {
              name:'currentUtilizationStatus',
              values:["NOT_ENOUGH_DATA"]
            }
          ]
        }
      }
      response_recommendations = rightsizing_recommendations(args_get_recommendations)
      response_recommendations[:edges].each do |edge|
        expect((edge[:node][:currentMonthlyPrice]).eql?(edge[:node][:projectedMonthlyPrice]) && edge[:node][:projectedMonthlySavings].eql?(0.0)).to be true
        end
      end
    end
    end
