# Test cases related to getting the recommendations

require 'spec_helper'
require 'date'

module RightsizingFrameworkTest
  describe 'RightsizingFramework GetRecommendations' do
    include_context 'request metadata'

    # Test -1 : Validate if a user is able to get the terminate recommendation
    xit 'Validate the recommendation for system defined efficiency target and 7 days evaluation duration for termination of an instance' do
      args_get_recommendations = {
        assetType: 'AWS_RDS',
        efficiencyTarget: 'crn:0:rightsizing/AWS_RDS_EfficiencyTarget:system_avg',
        evaluationDuration: 'LAST_7_DAYS',
        returnAllInstances: true
      }
      response_recommendations = rightsizing_recommendations(args_get_recommendations)
      instance_id = 'rightsizing-database-do-not-delete'
      cpu_value = get_metrics_value_based_on_userInput(instance_id, response_recommendations, 'avg_cpu')
      freeable_mem_value = get_metrics_value_based_on_userInput(instance_id, response_recommendations,
                                                                'avg_freeable_mem')
      expected_termination_recommendation = cpu_value <= 1 && freeable_mem_value >= 99 ? true : false
      termination_recommendation = get_current_termination_recommendation_of_instance(instance_id,
                                                                                      response_recommendations, nil)
      unless termination_recommendation.eql?(expected_termination_recommendation)
        raise('Recommendations are not matching the expectation ')
      end
    end

    # Test - 2 : Validate if a user is able to getting proper classification for over target
    it 'Validate the rightsizing recommendation to check over target instance' do
      metrics = [
        {
          metricName: 'avg_cpu',
          aggregation: 'AVG',
          lowerRange: 20,
          upperRange: 60
        },
        {
          metricName: 'avg_freeable_mem',
          aggregation: 'AVG',
          lowerRange: 20,
          upperRange: 60
        }
      ]
      args = {
        assetType: 'AWS_RDS',
        metrics: metrics,
        name: 'Test_Delete_ET' + rand(20_000).to_s
      }
      response = create_rightsizing_efficiency_target(args)
      et_id = response[:efficiencyTarget][:id]
      args_get_recommendations = {
        assetType: 'AWS_RDS',
        efficiencyTarget: et_id,
        evaluationDuration: 'LAST_60_DAYS',
        returnAllInstances: true,
        filterOptions: {
          otherCriteria: [
            {
              name: 'currentUtilizationStatus',
              values: %w[OVER_TARGET]
            }
          ]
        }
      }
      response_recommendations = rightsizing_recommendations(args_get_recommendations)
      response_recommendations[:edges].each do |_edge|
        utilization_status_from_api = response_recommendations([:edge][:currentUtilizationStatus])
        cpu_value = get_metrics_value_based_on_userInput(instance_id, response_recommendations, 'avg_cpu')
        expected_utilization_status_cpu = get_expected_utilization_status(cpu_value, lower_range, upper_range)
        freeable_mem_value = get_metrics_value_based_on_userInput(instance_id, response_recommendations, 'avg_cpu')
        expected_utilization_status_freeable = get_expected_utilization_status(cpu_value, lower_range, upper_range)
        raise('Recommendation status is not coming as expected')
        next if (expected_utilization_status_cpu.eql?('OVER_TARGET') && expected_utilization_status_freeable.eql?('GOOD_FIT')) ||
          (expected_utilization_status_cpu.eql?('GOOD_FIT') && expected_utilization_status_freeable.eql?('OVER_TARGET')) ||
          (expected_utilization_status_cpu.eql?('GOOD_FIT') && expected_utilization_status_freeable.eql?('GOOD_FIT'))

        # Clean up of the ET
        delete_ET(et_id)
      end
    end

    # Test - 3 : Validate the utilization status of any instance
    xit 'Validate utilization status for any instance' do
      lower_range = 20
      upper_range = 60
      metrics = [
        {
          metricName: 'avg_cpu',
          aggregation: 'AVG',
          lowerRange: lower_range,
          upperRange: upper_range
        }
      ]
      args = {
        assetType: 'AWS_RDS',
        metrics: metrics,
        name: 'Test_Delete_ET' + rand(20_000).to_s
      }
      response = create_rightsizing_efficiency_target(args)
      et_id = response[:efficiencyTarget][:id]
      args_get_recommendations = {
        assetType: 'AWS_RDS',
        efficiencyTarget: et_id,
        evaluationDuration: 'LAST_60_DAYS',
        returnAllInstances: true
      }
      response_recommendations = rightsizing_recommendations(args_get_recommendations)
      utilization_status_from_api = response_recommendations[:edges][0][:node][:currentUtilizationStatus]
      instance_id = get_instance_id(response_recommendations[:edges][0])
      cpu_value = get_metrics_value_based_on_userInput(instance_id, response_recommendations, 'avg_cpu')
      expected_utilization_status = get_expected_utilization_status(cpu_value, lower_range, upper_range)
      raise('Utilization status is not matching') unless utilization_status_from_api.eql?(expected_utilization_status)

      # Clean up of the ET
      delete_ET(et_id)
    end

    #  Test - 4 : Validate that the savings is the difference of the current and the projected price
    xit 'Validate saving of an instance is the difference of current price and the estimated price' do
      args_get_recommendations = {
        assetType: 'AWS_RDS',
        efficiencyTarget: 'crn:0:rightsizing/AWS_RDS_EfficiencyTarget:system_avg',
        evaluationDuration: 'LAST_30_DAYS',
        returnAllInstances: true
      }
      response_recommendations = rightsizing_recommendations(args_get_recommendations)
      projectedMonthlyCost = response_recommendations[:edges][0][:node][:projectedMonthlyPrice]
      currentMonthlyCost = response_recommendations[:edges][0][:node][:currentMonthlyPrice]
      # check with Srivathsan expected is not matching actual
      unless (currentMonthlyCost - projectedMonthlyCost).round(2).eql?((response_recommendations[:edges][0][:node][:projectedMonthlySavings]).round(2))
        raise('Data is not matching')
      end
    end

    # Test - 5 : Validate the total number of recommendations for an instance
    it 'Validate the number of recommendations for an instance' do
      args_get_recommendations = {
        assetType: 'AWS_RDS',
        efficiencyTarget: 'crn:0:rightsizing/AWS_RDS_EfficiencyTarget:system_avg',
        evaluationDuration: 'LAST_7_DAYS',
        returnAllInstances: true
      }
      response_recommendations = rightsizing_recommendations(args_get_recommendations)
      response_recommendations[:edges].each do |edge|
        raise('More than three recommendations are available') if edge[:node][:options].length > 3
      end
    end

    # Test - 6 : Validate best recommendation price is the lowest among the recommendation options
    it 'Validate the first best recommendation is of the lowest cost among the recommendations' do
      lower_range = 20
      upper_range = 50
      metrics = [
        {
          metricName: 'avg_cpu',
          aggregation: 'AVG',
          lowerRange: lower_range,
          upperRange: upper_range
        }
      ]
      args = {
        assetType: 'AWS_RDS',
        metrics: metrics,
        name: 'Test_Delete_ET' + rand(20_000).to_s
      }
      response = create_rightsizing_efficiency_target(args)
      et_id = response[:efficiencyTarget][:id]
      args_get_recommendations = {
        assetType: 'AWS_RDS',
        efficiencyTarget: et_id,
        evaluationDuration: 'LAST_7_DAYS',
        returnAllInstances: true
      }
      required_node_options = []
      response_recommendations = rightsizing_recommendations(args_get_recommendations)
      response_recommendations[:edges].each do |edge|
        if edge[:node][:options].length > 1 && edge[:node][:options].length <= 3
          required_node_options = edge[:node][:options]
          break
        end
      end
      recommendations_price_array = []
      first_best_recommendation_cost = ''
      unless required_node_options.size < 2
        required_node_options.each do |option|
          if option[:projectedMonthlyPrice] && option[:bestFit].eql?(true)
            first_best_recommendation_cost = option[:projectedMonthlyPrice]
            recommendations_price_array.append(first_best_recommendation_cost)
          elsif option[:projectedMonthlyPrice]
            recommendations_price_array.append(option[:projectedMonthlyPrice])
          end
        end
        recommendations_price_array = recommendations_price_array.sort
        unless first_best_recommendation_cost.eql?(recommendations_price_array[0])
          raise('Best fit recommendation is not the lowest cost recommendation')
        end
      end
      # Clean up of the ET
      delete_ET(et_id)
    end

    # Test-7 - Validate Projected savings must be based on the first best recommendation available.
    xit 'Validate Projected savings must be based on the first best recommendation available' do
      lower_range = 20
      upper_range = 50
      metrics = [
        {
          metricName: 'avg_freeable_mem',
          aggregation: 'AVG',
          lowerRange: lower_range,
          upperRange: upper_range
        }
      ]
      args = {
        assetType: 'AWS_RDS',
        metrics: metrics,
        name: 'Test_Delete_ET' + rand(20_000).to_s
      }
      response = create_rightsizing_efficiency_target(args)
      et_id = response[:efficiencyTarget][:id]
      args_get_recommendations = {
        assetType: 'AWS_RDS',
        efficiencyTarget: et_id,
        evaluationDuration: 'LAST_60_DAYS',
        returnAllInstances: true
      }
      response_recommendations = rightsizing_recommendations(args_get_recommendations)
      first_best_recommendation_price = 0
      price_from_api = 0
      response_recommendations[:edges].each do |edge|
        next unless edge[:node][:options].length >= 1

        edge[:node][:options].each do |option|
          next unless option[:bestFit].eql?(true)

          first_best_recommendation_price = option[:projectedMonthlyPrice]
          break
        end
        price_from_api = edge[:node][:projectedMonthlyPrice]
        break
      end
      raise('price not matching') unless first_best_recommendation_price.round(2).eql?(price_from_api.round(2))

      # Clean up of the ET
      delete_ET(et_id)
    end

    # Test-8 Validate whether the instances get listed if the selected metrics are not available for that and the recommendations are still being listed
    # No metrics available for many instances for the ET used in the test case, hence the expected recommendation must be->no recommendations
    xit 'Validate whether the instances get listed if the selected metrics are not available for that and the recommendations are still being listed', monitoring: true do
      args_get_recommendations = {
        assetType: 'AWS_RDS',
        efficiencyTarget: 'crn:0:rightsizing/AWS_RDS_EfficiencyTarget:system_avg',
        evaluationDuration: 'PREV_MONTH',
        returnAllInstances: true
      }
      response_recommendations = rightsizing_recommendations(args_get_recommendations)
      response_recommendations[:edges].each do |edge|
        if edge[:node][:options].length == 0 && edge[:node][:currentUtilizationStatus].eql?('NOT_ENOUGH_DATA')
          break
        elsif edge[:node][:options].length != 0 && edge[:node][:currentUtilizationStatus].eql?('NOT_ENOUGH_DATA')
          raise('Recommendations are available even though metrics are not are available for some instances')
        end
      end
      # Clean up of the ET
      delete_ET(et_id)
    end

    #test 9
    it 'Validate the utilisation summary from the API' do
      metrics = [
        {
          metricName: 'max_cpu',
          aggregation: 'MAX',
          lowerRange: 20,
          upperRange: 60
        },
        {
          metricName: 'avg_freeable_mem',
          aggregation: 'AVG',
          lowerRange: 20,
          upperRange: 60
        }
      ]
      args = {
        assetType: 'AWS_RDS',
        metrics: metrics,
        name: 'Test_Delete_ET' + rand(20_000).to_s
      }
      response = create_rightsizing_efficiency_target(args)
      et_id = response[:efficiencyTarget][:id]
      args_get_recommendations = {
        assetType: 'AWS_RDS',
        efficiencyTarget: et_id,
        evaluationDuration: 'LAST_60_DAYS',
        returnAllInstances: true,
        after: nil,
        sortRules: [
          {
            "field": "projectedMonthlyPrice",
            "direction": "ASC"
          }
        ]
      }
      result_map = {}
      current_price_calculated = 0
      projected_price_calculated = 0

      response_recommendations_call_1 = rightsizing_recommendations(args_get_recommendations)

      result_map = get_map_from_responses(response_recommendations_call_1)
      while response_recommendations_call_1[:pageInfo][:hasNextPage].eql?(true)
        args_get_recommendations = nil
        after_string_for_cursor = response_recommendations_call_1[:pageInfo][:endCursor]
        args_get_recommendations = {
          assetType: 'AWS_RDS',
          efficiencyTarget: et_id,
          evaluationDuration: 'LAST_60_DAYS',
          returnAllInstances: true,
          after: after_string_for_cursor,
          sortRules: [
            {
              "field": "projectedMonthlyPrice",
              "direction": "ASC"
            }
          ]
        }
        response_recommendations_call_1 = rightsizing_recommendations(args_get_recommendations)
        result_map = result_map.merge!(get_map_from_responses(response_recommendations_call_1))
      end
      args_get_summary = {
        assetType: 'AWS_RDS',
        efficiencyTarget: et_id,
        evaluationDuration: 'LAST_60_DAYS',
        returnAllInstances: true
      }
      response_summary = rightsizing_recommendations_summary(args_get_summary)
      result_map.each_value do |edge|
        current_price_calculated += edge[:node][:currentMonthlyPrice]
        projected_price_calculated += edge[:node][:projectedMonthlyPrice]
      end
      current_savings_calculated = current_price_calculated - projected_price_calculated
      saving_value_from_api = response_summary[:projectedMonthlyDifference]
      is_savings_matching = current_savings_calculated.round.eql?(saving_value_from_api.round)
      is_projected_price_matching = projected_price_calculated.round.eql?(response_summary[:projectedMonthlyPrice].round)
      is_current_price_matching = current_price_calculated.round.eql?(response_summary[:monthlyCost].round)
      unless is_savings_matching && is_projected_price_matching && is_current_price_matching
        raise('Summary is not matching')
      end

      delete_ET(et_id)
    end

    # Test 10 verify filter by perspective
    it 'Validate filter by perspective', monitoring: true do
      expected_instance_recommendation_map = {}
      expected_instance_recommendation_map.store('do-not-delete-rightsizing-test', 'db.t4g.small')
      # expected_instance_recommendation_map.store('do-not-delete-rightsizing-test-3','terminate')
      args_get_recommendations = {
        assetType: 'AWS_RDS',
        efficiencyTarget: 'crn:0:rightsizing/AWS_RDS_EfficiencyTarget:system_avg',
        evaluationDuration: 'LAST_7_DAYS',
        filterOptions: {
          perspectives:
            [{
               perspectiveId: '5772436099236',
               groupIds: [
                 5772436888312,
                 5772436888317
               ]
             }]
        },
        returnAllInstances: true
      }
      response_recommendations_call = rightsizing_recommendations(args_get_recommendations)
      map_1 = get_map_from_responses(response_recommendations_call)
      expected_instance_recommendation_map.each_key do |instance_id|
        edge = map_1[instance_id]
        options = get_recommendation_for_anInstance(edge)
        termination_recommendation = get_current_termination_recommendation_of_instance(instance_id, nil, edge)
        api_response = Hash.new
        if termination_recommendation.eql?(true) && options.size.eql?(0)
          api_response.store(instance_id, 'terminate')
        elsif termination_recommendation.eql?(true) && !options.size.eql?(0)
          raise('no terminate recommendation, no recommendations')
        elsif termination_recommendation.eql?(false) && options.size.eql?(0)
          api_response.store(instance_id, '')
        elsif termination_recommendation.eql?(false) && !options.size.eql?(0)
          first_best_rec = get_first_best_recommendation(instance_id, nil, edge)
          api_response.store(instance_id, first_best_rec)
        end
        fail("Data not matching with baseline data for recommendations, expected is : #{expected_instance_recommendation_map} and actual is : #{api_response}") unless expected_instance_recommendation_map == api_response
      end
      # could also fail in case there is deletion of instance of perspective starts including more instances
      # if response_recommendations_call[:edges].size >= 16
      #   raise('all the instances are returning')
      # end
    end

    # Test 11 Verify filter by availability zone
    it 'Validate filter by availability zone', monitoring: true do
      expected_instance_recommendation_map = {}
      expected_instance_recommendation_map.store('do-not-delete-rightsizing-test', 'db.t4g.small')
      expected_instance_recommendation_map.store('mj-db-test-instance-1', '')
      args_get_recommendations = {
        assetType: 'AWS_RDS',
        efficiencyTarget: 'crn:0:rightsizing/AWS_RDS_EfficiencyTarget:system_avg',
        evaluationDuration: 'LAST_7_DAYS',
        filterOptions:
          {
            otherCriteria: [
              {
                name: 'aws_availability_zone_id',
                values: ['us-east-1a']
              }
            ]
          },
        returnAllInstances: true
      }
      response_recommendations_call = rightsizing_recommendations(args_get_recommendations)
      map_1 = get_map_from_responses(response_recommendations_call)
      expected_instance_recommendation_map.each_key do |instance_id|
        edge = map_1[instance_id]
        options = get_recommendation_for_anInstance(edge)
        best_recommendation_1 = get_first_best_recommendation(instance_id, nil, edge)
        termination_recommendation = get_current_termination_recommendation_of_instance(instance_id, nil, edge)
        if termination_recommendation.eql?(true) && options.size.eql?(0)
          raise('getting terminate recommendation, not same as expected')
        elsif termination_recommendation.eql?(true) && !options.size.eql?(0)
          raise('Getting recommendation even with termination')
        elsif termination_recommendation.eql?(false) && options.size.eql?(0)
          raise('Different recommendation') unless instance_id.eql?('mj-db-test-instance-1')
        elsif termination_recommendation.eql?(false) && !options.size.eql?(0)
          first_best_rec = get_first_best_recommendation(instance_id, nil, edge)
          if first_best_rec != expected_instance_recommendation_map[instance_id]
            raise('Not getting the expected recommendation')
          end
        end
      end
      # could also fail in case there is deletion of instance of perspective starts including more instances
      if response_recommendations_call[:edges].size <0
        raise('instances are not getting filtered properly issue 4 instances are not getting returned')
      end
    end

    # test 12 Verify filter by combination of 2
    xit 'Validate filter by combination of two filter' do
      expected_instance_recommendation_map = {}
      expected_instance_recommendation_map.store('do-not-delete-rightsizing-test', 'db.t4g.small')
      expected_instance_recommendation_map.store('mj-db-test-instance-1', '')
      args_get_recommendations = {
        assetType: 'AWS_RDS',
        efficiencyTarget: 'crn:0:rightsizing/AWS_RDS_EfficiencyTarget:system_avg',
        evaluationDuration: 'LAST_7_DAYS',
        filterOptions:
          {
            perspectives:
              [
                {
                  perspectiveId: 5_772_436_099_236,
                  groupIds: [5_772_436_692_381, 5_772_436_692_382, 5_772_436_692_380]
                }
              ],
            otherCriteria:
              [
                {
                  name:
                    'aws_availability_zone_id',
                  values:
                    ['us-east-1a']
                }
              ]
          },
        returnAllInstances: true
      }
      response_recommendations_call = rightsizing_recommendations(args_get_recommendations)
      map_1 = get_map_from_responses(response_recommendations_call)
      expected_instance_recommendation_map.each_key do |instance_id|
        edge = map_1[instance_id]
        options = get_recommendation_for_anInstance(edge)
        best_recommendation_1 = get_first_best_recommendation(instance_id, nil, edge)
        termination_recommendation = get_current_termination_recommendation_of_instance(instance_id, nil, edge)
        if termination_recommendation.eql?(true) && options.size.eql?(0)
          raise('getting terminate recommendation, not same as expected')
        elsif termination_recommendation.eql?(true) && !options.size.eql?(0)
          raise('Getting recommendation even with termination')
        elsif termination_recommendation.eql?(false) && options.size.eql?(0)
          raise('Different recommendation') unless instance_id.eql?('mj-db-test-instance-1')
        elsif termination_recommendation.eql?(false) && !options.size.eql?(0)
          first_best_rec = get_first_best_recommendation(instance_id, nil, edge)
          if first_best_rec != expected_instance_recommendation_map[instance_id]
            raise('Not getting the expected recommendation')
          end
        end
      end
      # could also fail in case there is deletion of instance of perspective starts including more instances
      if response_recommendations_call[:edges].size != 4
        raise('instances are not getting filtered properly issue 4 instances are not getting returned')
      end
    end

    # Test 13 showALL true is giving some no recommendation response
    xit 'Validate show ALL is true' do
      expected_instance_recommendation_map = {}
      instance_id = 'do-not-delete-rightsizing-test-4'
      args_get_recommendations = {
        assetType: 'AWS_RDS',
        efficiencyTarget: 'crn:0:rightsizing/AWS_RDS_EfficiencyTarget:system_avg',
        evaluationDuration: 'LAST_7_DAYS',
        returnAllInstances: true
      }
      response_recommendations_call = rightsizing_recommendations(args_get_recommendations)
      map_1 = get_map_from_responses(response_recommendations_call)
      edge = map_1[instance_id]
      options = get_recommendation_for_anInstance(edge)
      termination_recommendation = get_current_termination_recommendation_of_instance(instance_id, nil, edge)
      if termination_recommendation.eql?(true) && !options.size.eql?(0)
        raise('not expected recommendation')
      elsif termination_recommendation.eql?(false) && options.size.eql?(0)
        raise('no recommendation')
      end
    end

    # Test 14 ShowALL false does not have any showAll false
    it 'Validate showALL is false the instance with no recommendation should not be present', monitoring: true do
      expected_instance_recommendation_map = {}
      instance_id = 'do-not-delete-rightsizing-test-4'
      args_get_recommendations = {
        assetType: 'AWS_RDS',
        efficiencyTarget: 'crn:0:rightsizing/AWS_RDS_EfficiencyTarget:system_avg',
        evaluationDuration: 'LAST_7_DAYS',
        returnAllInstances: false
      }
      response_recommendations_call = rightsizing_recommendations(args_get_recommendations)
      map_1 = get_map_from_responses(response_recommendations_call)
      map_1.each_value do |edge|
        instanceID = get_instance_id(edge)
        raise('instance with no recommendation should not be present') if instance_id == instanceID
      end
    end

    # Test 15 recommend burstable true has burstable recommmended- TO DO :OP-1896
    xit 'Validate recommendations for an instance with recommend burstables flag as true' do
      expected_instance_recommendation_map = {}
      expected_instance_recommendation_map.store('do-not-delete-rightsizing-test', 'db.t4g.medium')
      expected_instance_recommendation_map.store('do-not-delete-rightsizing-test-3', 'db.t4g.medium')
      args_get_recommendations = {
        assetType: 'AWS_RDS',
        efficiencyTarget: 'crn:0:rightsizing/AWS_RDS_EfficiencyTarget:system_avg',
        evaluationDuration: 'LAST_7_DAYS',
        processingFlagsMap: [
          {
            key: 'recommendBurstable',
            value: 'true'
          }
        ],
        returnAllInstances: true
      }
      response_recommendations_call = rightsizing_recommendations(args_get_recommendations)
      api_response = {}
      map_1 = get_map_from_responses(response_recommendations_call) # key - instance_id, value - edge of an instance
      expected_instance_recommendation_map.each_key do |instance_id|
        edge = map_1[instance_id]
        options = get_recommendation_for_anInstance(edge)
        next unless options.size > 0

        recommendation = get_first_best_recommendation(instance_id, nil, edge)
        if recommendation != expected_instance_recommendation_map[instance_id]
          raise('Not getting burstable recommendation')
        end
      end
    end

    # Test 16 recommend burstable false
    it 'Validate recommendations for an instance with recommend burstables flag as false', monitoring: true do
      expected_instance_recommendation_map = {}
      expected_instance_recommendation_map.store('do-not-delete-rightsizing-test', 'db.t4g.small')
      expected_instance_recommendation_map.store('do-not-delete-rightsizing-test-3', 'db.t4g.medium')
      args_get_recommendations = {
        assetType: 'AWS_RDS',
        efficiencyTarget: 'crn:0:rightsizing/AWS_RDS_EfficiencyTarget:system_avg',
        evaluationDuration: 'LAST_7_DAYS',
        processingFlagsMap: [
          {
            key: 'recommendBurstable',
            value: 'false'
          }
        ],
        returnAllInstances: true
      }
      response_recommendations_call = rightsizing_recommendations(args_get_recommendations)
      api_response = {}
      map_1 = get_map_from_responses(response_recommendations_call) # key - instance_id, value - edge of an instance
      expected_instance_recommendation_map.each_key do |instance_id|
        edge = map_1[instance_id]
        options = get_recommendation_for_anInstance(edge)
        next unless options.size > 0

        recommendation = get_first_best_recommendation(instance_id, nil, edge)
        if recommendation = expected_instance_recommendation_map[instance_id] && recommendation.start_with?('t')
          raise('Not getting burstable recommendation')
        end
      end
    end

    # Test 17 Pagination
    it 'verify pagination', monitoring: true do
      expected_instance_recommendation_map = {}
      args_get_recommendations = {
        assetType: 'AWS_RDS',
        efficiencyTarget: 'crn:0:rightsizing/AWS_RDS_EfficiencyTarget:system_avg',
        evaluationDuration: 'LAST_7_DAYS',
        processingFlagsMap: [
          {
            key: 'recommendBurstable',
            value: 'true'
          }
        ],
        returnAllInstances: true,
        first:9
      }
      response_recommendations_call = rightsizing_recommendations(args_get_recommendations)
      raise('pagination is failing with first') if response_recommendations_call[:edges].size > 9
      if response_recommendations_call[:pageInfo][:hasNextPage].eql?(true)
        after_string_for_cursor = response_recommendations_call[:pageInfo][:endCursor]
        args_get_recommendations_1 = {
          assetType: 'AWS_RDS',
          efficiencyTarget: 'crn:0:rightsizing/AWS_RDS_EfficiencyTarget:system_avg',
          evaluationDuration: 'LAST_7_DAYS',
          processingFlagsMap: [
            {
              key: 'recommendBurstable',
              value: 'true'
            }
          ],
          returnAllInstances: true,
          first:9,
          after: after_string_for_cursor
        }
        response_recommendations_call_1 = rightsizing_recommendations(args_get_recommendations_1)
        fail ('no edges returned on applying after cursor') if response_recommendations_call_1[:edges].size.eql?(0)
      end
    end

    # Multiple filters
    it 'Validate the filter with multiple utilization status' do
      other_criteria = [
        {
          name: 'currentUtilizationStatus',
          values: %w[GOOD_FIT UNDER_TARGET]
        }
      ]
      args_get_recommendations = {
        assetType: 'AWS_RDS',
        efficiencyTarget: 'crn:0:rightsizing/AWS_RDS_EfficiencyTarget:system_max',
        evaluationDuration: 'LAST_30_DAYS',
        filterOptions: {
          otherCriteria: other_criteria
        },
        returnAllInstances: true
      }
      response_recommendations_call_1 = rightsizing_recommendations(args_get_recommendations)
      expect(response_recommendations_call_1[:edges].all? { |r| other_criteria[0][:values].include? r[:node][:currentUtilizationStatus] }).to be true
    end



    # Method to get the days in a month
    COMMON_YEAR_DAYS_IN_MONTH = [nil, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31]

    def days_in_month(month, year = Time.now.year)
      return 29 if month == 2 && Date.gregorian_leap?(year)

      COMMON_YEAR_DAYS_IN_MONTH[month]
    end

    # Method to get the expected utilization status
    def get_expected_utilization_status(current_value, lower_range, upper_range)
      status = ''
      if current_value.eql?(nil)
      status = 'NOT_ENOUGH_DATA'
      elsif current_value < lower_range
        status = 'UNDER_TARGET'
      elsif current_value > upper_range
        status = 'OVER_TARGET'
      elsif current_value >= lower_range && current_value <= upper_range
        status = 'GOOD_FIT'
      end
      status
    end

    # Map contains key as instance id and value as the data of each node
    def get_map_from_responses(recommendations_response)
      responses_map = {}
      recommendations_response[:edges].each do |edge|
        responses_map.store(get_instance_id(edge), edge)
      end
      responses_map
    end


    # Returns the current metrics value of the instance passed and the metric id
    def get_metrics_value_based_on_userInput(instance_id, recommendation_response, metricId)
      current_metrics = get_current_metrics(instance_id, recommendation_response)
      val = 0
      current_metrics.each do |metric|
        val = metric[:value] if metric[:metricName].eql?(metricId)
      end
      val
    end

    # Returns the current metrics status of the instance passed
    def get_metrics_status_based_on_userInput(instance_id, recommendation_response, metricId)
      current_metrics = get_current_metrics(instance_id, recommendation_response)
      status = ''
      current_metrics.each do |metric|
        status = metric[:status] if metric[:metricName].eql?(metricId)
      end
      status
    end

    # Returns the current metrics of the instance passed
    def get_current_metrics(instance_id, recommendation_response)
      map = {}
      recommendation_response[:edges].each do |edge|
        map.store(get_instance_id(edge), edge[:node][:currentMetrics])
      end
      map[instance_id]
    end

    # Returns the termination recommendation of the instance
    def get_current_termination_recommendation_of_instance(instance_id, recommendation_response, edge)
      if !recommendation_response.eql?(nil)
        instance_values_fromMap = get_details(instance_id, recommendation_response)
        instance_values_fromMap['terminateRecommendation']
      elsif recommendation_response.eql?(nil) && !edge.eql?(nil)
        edge[:node][:terminateRecommendation]
      end
    end

    # Method to get the recommendations for an instance
    def get_recommendation_for_anInstance(edge)
      edge[:node][:options]
    end

    # Returns the summary of the instance passed
    def get_details(instance_id, response)
      map_data = get_data_from_response(response)
      map_data[instance_id]
    end

    # Returns map of key value pair where key is the instance id and value is the another hashmap of utilization status, termination recommendation and recommendation options.
    def get_data_from_response(recommendations_response)
      map = {}
      recommendations_response[:edges].each do |edge|
        data_list = {}
        data_list.store('currentUtilizationStatus', edge[:node][:currentUtilizationStatus])
        data_list.store('terminateRecommendation', edge[:node][:terminateRecommendation])
        data_list.store('options', edge[:node][:options])
        map.store(get_instance_id(edge), data_list)
      end
      map
    end

    # Returns the instance id of the node passed
    def get_instance_id(edge)
      instance_id = edge[:node][:targetAsset][:awsInstanceId]
      return instance_id
    end


    # Returns the first best recommendation of the instance if there is any recommendation
    def get_first_best_recommendation(instance_id, recommendation_response, edge)
      if !recommendation_response.eql?(nil)
        instance_values_fromMap = get_details(instance_id, recommendation_response)
        recommendation_options = instance_values_fromMap['options']
        first_best_recommendation = ''
        recommendation_options.each do |option|
          first_best_recommendation = option[:instanceType][:name] if option[:bestFit].eql?(true)
        end
        first_best_recommendation
      elsif recommendation_response.eql?(nil) && !edge.eql?(nil)
        if edge[:node][:options].size.eql?(0)
          'No recommendations'
        else
          edge[:node][:options].each do |option|
            return option[:instanceType][:name] if option[:bestFit].eql?(true)
          end
        end
      end
    end

    # Returns the summary of the instance passed
    def get_details(instance_id, response)
      map_data = get_data_from_response(response)
      map_data[instance_id]
    end

    # Returns map of key value pair where key is the instance id and value is the another hashmap of utilization status, termination recommendation and recommendation options.
    def get_data_from_response(recommendations_response)
      map = {}
      recommendations_response[:edges].each do |edge|
        data_list = {}
        data_list.store('currentUtilizationStatus', edge[:node][:currentUtilizationStatus])
        data_list.store('terminateRecommendation', edge[:node][:terminateRecommendation])
        data_list.store('options', edge[:node][:options])
        map.store(get_instance_id(edge), data_list)
      end
      map
    end
  end
end
