require 'spec_helper'

module RightsizingFrameworkTest
  describe 'RightsizingFramework GroupBY' do
    include_context 'request metadata'

    # Test -1 Group by API ACCOUNT RDS
    xit 'Validate group by API account RDS ' do
      id= "844584796563"
      args_get_summary = {
        assetType: 'AWS_RDS',
        efficiencyTarget: 'crn:0:rightsizing/AWS_RDS_EfficiencyTarget:system_avg',
        evaluationDuration: 'LAST_7_DAYS',
        returnAllInstances: true,
        filterOptions: {
          otherCriteria:
          [
            {
              name: "aws_account_id",
              values: [
                id
              ]
            }
          ]
        }
      }
      rightsizing_recommendations_summary(args_get_summary)
      summary_det=JSON.parse(response.body)
      expected_currentMonthlyPrice=(summary_det["data"]["rightsizingRecommendationsSummary"]["monthlyCost"]).round(2)
      expected_projectedMonthlyPrice=(summary_det["data"]["rightsizingRecommendationsSummary"]["projectedMonthlyPrice"]).round(2)
      expected_projectedMonthlySavings=(summary_det["data"]["rightsizingRecommendationsSummary"]["projectedMonthlyDifference"]).round(2)
      args_get_recommendations = {
        assetType: 'AWS_RDS',
        efficiencyTarget: 'crn:0:rightsizing/AWS_RDS_EfficiencyTarget:system_avg',
        evaluationDuration: 'LAST_7_DAYS',
        returnAllInstances: true,
        groupingCriteria:
          {
			   type: 'ACCOUNT'
		      }
      }
      response=rightsizing_recommendations_aggregation(args_get_recommendations)
      node=response[:edges].select { |x| x[:node][:id].eql?(id) }[0][:node]
      expect(node[:name]).to eq("Redshirt") #account name and id will not change hence comparing with hardcoded value
      expect(((node[:currentMonthlyPrice].round(2))-expected_currentMonthlyPrice).abs).to be <(0.10)
      expect(((node[:projectedMonthlyPrice].round(2))-expected_projectedMonthlyPrice).abs).to be <(0.10)
      expect(((node[:projectedMonthlySavings].round(2))-expected_projectedMonthlySavings).abs).to be <(0.10) #https://cloudhealthtech.atlassian.net/browse/OP-1162
    end

    # Test -2 Group by API ACCOUNT EC2
    it 'Validate group by API account EC2' do
      id="844584796563"
      args_get_summary = {
        assetType: 'AWS_EC2',
        efficiencyTarget: 'crn:0:rightsizing/AWS_EC2_EfficiencyTarget:system_avg',
        evaluationDuration: 'LAST_7_DAYS',
        returnAllInstances: true,
        filterOptions: {
          otherCriteria:
            [
              {
                name: "aws_account_id",
                values: [
                  id
                ]
              }
            ]
        }
      }

      response=rightsizing_recommendations_summary(args_get_summary)
      expected_currentMonthlyPrice=response[:monthlyCost].round(2)
      expected_projectedMonthlyPrice=response[:projectedMonthlyPrice].round(2)
      expected_projectedMonthlySavings=response[:projectedMonthlyDifference].round(2)
      args_get_recommendations = {
        assetType: 'AWS_EC2',
        efficiencyTarget: 'crn:0:rightsizing/AWS_EC2_EfficiencyTarget:system_avg',
        evaluationDuration: 'LAST_7_DAYS',
        returnAllInstances: true,
        groupingCriteria:
          {
            type: 'ACCOUNT'
          }
      }
      response=rightsizing_recommendations_aggregation(args_get_recommendations)
      node=response[:edges].select { |x| x[:node][:id].eql?(id) }[0][:node]
      expect(node[:name]).to eq("Redshirt")
      expect((node[:currentMonthlyPrice]-expected_currentMonthlyPrice).abs).to be<(0.10)
      expect((node[:projectedMonthlyPrice]-expected_projectedMonthlyPrice).abs).to be<(0.10)
      expect((node[:projectedMonthlySavings]-expected_projectedMonthlySavings).abs).to be<(0.10)
    end

    # Test -3 Group by API perspective by EC2
    xit 'Validate group by API perspective for EC2' do #https://cloudhealthtech.atlassian.net/browse/OP-1162 - ticket to make the values in sync
      perspective_id = '5772436099236' #test perspective
      args_get_recommendations = {
        assetType: 'AWS_EC2',
        efficiencyTarget: 'crn:0:rightsizing/AWS_EC2_EfficiencyTarget:system_avg',
        evaluationDuration: 'LAST_7_DAYS',
        returnAllInstances: true,
        groupingCriteria:
          {
            type: 'PERSPECTIVE',
            value: perspective_id
          }
      }
      response=rightsizing_recommendations_aggregation(args_get_recommendations)
      id =response[:edges][0][:node][:id]
      args_get_summary = {
        assetType: 'AWS_EC2',
        efficiencyTarget: 'crn:0:rightsizing/AWS_EC2_EfficiencyTarget:system_avg',
        evaluationDuration: 'LAST_7_DAYS',
        returnAllInstances: true,
        filterOptions: {
          perspectives:
            [{
               perspectiveId: perspective_id,
               groupIds: [
                 id
               ]
             }]
        }
      }

      summary_response=rightsizing_recommendations_summary(args_get_summary)
      expected_currentMonthlyPrice=summary_response[:monthlyCost]
      expected_projectedMonthlyPrice=summary_response[:projectedMonthlyPrice]
      expected_projectedMonthlySavings=summary_response[:projectedMonthlyDifference]
      expect(response[:edges][0][:node][:name]).to eq("Amazon EC2 Instance active")
      expect((response[:edges][0][:node][:currentMonthlyPrice]-expected_currentMonthlyPrice).abs).to be <(0.10)
      expect((response[:edges][0][:node][:projectedMonthlyPrice]-expected_projectedMonthlyPrice).abs).to be <(0.10)
      expect((response[:edges][0][:node][:projectedMonthlySavings]-expected_projectedMonthlySavings).abs).to be <(0.10)

    end

    # Test -4 Group by API perspective by RDS
    xit 'Validate group by API perspective RDS' do
      perspective_id = '5772436099236' #test perspective created to test rz
      variables = {
        assetType: 'AWS_RDS',
        efficiencyTarget: 'crn:0:rightsizing/AWS_RDS_EfficiencyTarget:system_avg',
        evaluationDuration: 'LAST_7_DAYS',
        returnAllInstances: true,
        groupingCriteria:
          {
            type: 'PERSPECTIVE',
            value: perspective_id
          }
      }
      response=rightsizing_recommendations_aggregation(variables)
      id=response[:edges][0][:node][:id]
      args_get_summary = {
        assetType: 'AWS_RDS',
        efficiencyTarget: 'crn:0:rightsizing/AWS_RDS_EfficiencyTarget:system_avg',
        evaluationDuration: 'LAST_7_DAYS',
        returnAllInstances: true,
        filterOptions: {
          perspectives:
            [{
               perspectiveId: perspective_id,
               groupIds: [
                 id
               ]
             }]
        }
      }
      summary_response=rightsizing_recommendations_summary(args_get_summary)
      expected_currentMonthlyPrice=summary_response[:monthlyCost].round(2)
      expected_projectedMonthlyPrice=summary_response[:projectedMonthlyPrice].round(2)
      expected_projectedMonthlySavings=summary_response[:projectedMonthlyDifference].round(2)
      expect(response[:edges][0][:node][:name]).to eq('active instances')
      expect((response[:edges][0][:node][:currentMonthlyPrice]-expected_currentMonthlyPrice).abs).to be <(0.10)
      expect((response[:edges][0][:node][:projectedMonthlyPrice]-expected_projectedMonthlyPrice).abs).to be <(0.10)
      expect((response[:edges][0][:node][:projectedMonthlySavings]-expected_projectedMonthlySavings).abs).to be <(0.10)
    end

    # Test -4 Group by API perspective by AzureVM
    xit 'Validate group by subscription saving value and filter saving value' do
      args_get_recommendations =
        {
            assetType: "AZURE_VM",
            efficiencyTarget: "crn:0:rightsizing/AZURE_VM_EfficiencyTarget:system_avg",
            evaluationDuration: "LAST_7_DAYS",
            processingFlagsMap: [
              {
                key: "recommendBurstable",
                value: "true"
              }
            ],
            returnAllInstances: true,
            groupingCriteria: {
              type: "SUBSCRIPTION"
            },
          sortRules: [
            {
              field: "projectedMonthlySavings",
              direction: "DESC"
            }
          ]
        }
      response=rightsizing_recommendations_aggregation(args_get_recommendations)
      id='700f3a5c-8c56-44b9-aa1d-fcfb1493aa4b'
      node=response[:edges].select { |x| x[:node][:id].eql?(id) }[0][:node]
      group_current_price=node[:currentMonthlyPrice].round(2)
      group_projected_price=node[:projectedMonthlyPrice].round(2)
      group_saving=node[:projectedMonthlySavings].round(2)
      args_get_summary={
          assetType: "AZURE_VM",
          efficiencyTarget: "crn:0:rightsizing/AZURE_VM_EfficiencyTarget:system_avg",
          evaluationDuration: "LAST_7_DAYS",
          filterOptions: {
            otherCriteria: [
              {
                name: "azure_subscription_id",
                values: [
                  "700f3a5c-8c56-44b9-aa1d-fcfb1493aa4b"
                ]
              }
            ]
          },
          processingFlagsMap: [
            {
              key: "recommendBurstable",
              value: "true"
            }
          ]
        }
      response_recommendation= rightsizing_recommendations_summary(args_get_summary)
      expected_summary=response_recommendation[:monthlyCost].round(2)
      expected_monthlyPrice=response_recommendation[:projectedMonthlyPrice].round(2)
      expected_saving=response_recommendation[:projectedMonthlyDifference].round(2)
      expect((group_current_price-expected_summary).abs).to be <(0.10)
      expect((group_projected_price-expected_monthlyPrice).abs).to be <(0.10)
      expect((group_saving-expected_saving).abs).to be <(0.10)
    end

    #verify all the data is returned in case of instances marked as other
    xit 'Validating unassigned assets listed in other in group by' do
      args_get_recommendations =
        {
          assetType: "AZURE_VM",
          efficiencyTarget: "crn:0:rightsizing/AZURE_VM_EfficiencyTarget:system_avg",
          evaluationDuration: "LAST_7_DAYS",
          processingFlagsMap: [
            {
              key: "recommendBurstable",
              value: true
            }
          ],
          returnAllInstances: true,
          groupingCriteria: {
            type: "PERSPECTIVE",
            value:'5772436099236'
          },
          sortRules: [
            {
              field: "projectedMonthlySavings",
              direction: "DESC"
            }
          ]
        }
      response=rightsizing_recommendations_aggregation(args_get_recommendations)
      expect((response[:edges].select { |x| x[:node][:name].eql?('Other') }).size).to be>0
    end

    it 'Filter by perspective testing for s3' do #perspective is created for testing purpose for rightsizing
      perspective_id = '168'
      group_id= '938'
      args_get_summary={
        assetType: "AWS_S3",
        efficiencyTarget: "crn:0:rightsizing/AWS_S3_EfficiencyTarget:system",
        evaluationDuration: "LAST_4_WEEKS",
        filterOptions: {
          perspectives: [
            {
              perspectiveId: perspective_id,
              groupIds: [
                group_id
              ]
            }
          ]
        },
        processingFlagsMap: [
          {
            key: "latencyMatch",
            value: true
          }
        ]
      }
      response_recommendation= rightsizing_recommendations_summary(args_get_summary)
      expected_summary=response_recommendation[:currentMonthAggregatedCost].round(2)
      expected_monthlyPrice=response_recommendation[:currentMonthProjectedCost].round(2)
      expected_saving=response_recommendation[:projectedMonthlySavingsByCost].round(2)
      args_get_recommendations =
        {
          assetType: "AWS_S3",
          efficiencyTarget: "crn:0:rightsizing/AWS_S3_EfficiencyTarget:system",
          evaluationDuration: "LAST_4_WEEKS",
          processingFlagsMap: [
            {
              key: "recommendBurstable",
              value: true
            }
          ],
          returnAllInstances: true,
          groupingCriteria: {
            type: "PERSPECTIVE",
            value:'168'
          },
          sortRules: [
            {
              field: "projectedMonthlySavings",
              direction: "DESC"
            }
          ]
        }
      response=rightsizing_recommendations_aggregation(args_get_recommendations)
      expect((response[:edges].select { |x| x[:node][:id].eql?(group_id) }))
      expect((response[:edges].select { |x| x[:node][:currentCost].eql?(expected_summary) }))
      expect((response[:edges].select { |x| x[:node][:projectedCost].eql?(expected_monthlyPrice) }))
      expect((response[:edges].select { |x| x[:node][:projectedSavingsByCost].eql?(expected_saving) }))
    end
  end
  end